@@ -14,20 +14,24 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2018 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up a H.264 Video Elementary Stream into NAL units.
 // Implementation
 
 #include "H264VideoStreamFramer.hh"
 
 H264VideoStreamFramer* H264VideoStreamFramer
-::createNew(UsageEnvironment& env, FramedSource* inputSource, Boolean includeStartCodeInOutput) {
-  return new H264VideoStreamFramer(env, inputSource, True, includeStartCodeInOutput);
+::createNew(UsageEnvironment& env, FramedSource* inputSource,
+	    Boolean includeStartCodeInOutput, Boolean insertAccessUnitDelimiters) {
+  return new H264VideoStreamFramer(env, inputSource, True,
+				   includeStartCodeInOutput, insertAccessUnitDelimiters);
 }
 
 H264VideoStreamFramer
-::H264VideoStreamFramer(UsageEnvironment& env, FramedSource* inputSource, Boolean createParser, Boolean includeStartCodeInOutput)
-  : H264or5VideoStreamFramer(264, env, inputSource, createParser, includeStartCodeInOutput) {
+::H264VideoStreamFramer(UsageEnvironment& env, FramedSource* inputSource, Boolean createParser,
+			Boolean includeStartCodeInOutput, Boolean insertAccessUnitDelimiters)
+  : H264or5VideoStreamFramer(264, env, inputSource, createParser,
+			     includeStartCodeInOutput, insertAccessUnitDelimiters) {
 }
 
 H264VideoStreamFramer::~H264VideoStreamFramer() {
