@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2018 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up a H.264 or H.265 Video Elementary Stream into NAL units.
 // Implementation
 
@@ -74,9 +74,11 @@ private:
 
 H264or5VideoStreamFramer
 ::H264or5VideoStreamFramer(int hNumber, UsageEnvironment& env, FramedSource* inputSource,
-			   Boolean createParser, Boolean includeStartCodeInOutput)
+			   Boolean createParser,
+			   Boolean includeStartCodeInOutput, Boolean insertAccessUnitDelimiters)
   : MPEGVideoStreamFramer(env, inputSource),
-    fHNumber(hNumber),
+    fHNumber(hNumber), fIncludeStartCodeInOutput(includeStartCodeInOutput),
+    fInsertAccessUnitDelimiters(insertAccessUnitDelimiters),
     fLastSeenVPS(NULL), fLastSeenVPSSize(0),
     fLastSeenSPS(NULL), fLastSeenSPSSize(0),
     fLastSeenPPS(NULL), fLastSeenPPSSize(0) {
@@ -143,6 +145,40 @@ Boolean H264or5VideoStreamFramer::isVCL(u_int8_t nal_unit_type) {
     : (nal_unit_type <= 31);
 }
 
+void H264or5VideoStreamFramer::doGetNextFrame() {
+  if (fInsertAccessUnitDelimiters && pictureEndMarker()) {
+    // Deliver an "access_unit_delimiter" NAL unit instead:
+    unsigned const startCodeSize = fIncludeStartCodeInOutput ? 4: 0;
+    unsigned const audNALSize = fHNumber == 264 ? 2 : 3;
+
+    fFrameSize = startCodeSize + audNALSize;
+    if (fFrameSize > fMaxSize) { // there's no space
+      fNumTruncatedBytes = fFrameSize - fMaxSize;
+      fFrameSize = fMaxSize;
+      handleClosure();
+      return;
+    }
+
+    if (fIncludeStartCodeInOutput) {
+      *fTo++ = 0x00; *fTo++ = 0x00; *fTo++ = 0x00; *fTo++ = 0x01;
+    }
+    if (fHNumber == 264) {
+      *fTo++ = 9; // "Access unit delimiter" nal_unit_type
+      *fTo++ = 0xF0; // "primary_pic_type" (7); "rbsp_trailing_bits()"
+    } else { // H.265
+      *fTo++ = 35<<1; // "Access unit delimiter" nal_unit_type
+      *fTo++ = 0; // "nuh_layer_id" (0); "nuh_temporal_id_plus1" (0) (Is this correct??)
+      *fTo++ = 0x50; // "pic_type" (2); "rbsp_trailing_bits()" (Is this correct??)
+    }
+
+    pictureEndMarker() = False; // for next time
+    afterGetting(this);
+  } else {
+    // Do the normal delivery of a NAL unit from the parser:
+    MPEGVideoStreamFramer::doGetNextFrame();
+  }
+}
+
 
 ////////// H264or5VideoStreamParser implementation //////////
 
@@ -894,11 +930,11 @@ void H264or5VideoStreamParser
       unsigned dpb_output_delay = bv.getBits(dpb_output_delay_length_minus1 + 1);
       DEBUG_PRINT(dpb_output_delay);
     }
+    double prevDeltaTfiDivisor = DeltaTfiDivisor; 
     if (pic_struct_present_flag) {
       unsigned pic_struct = bv.getBits(4);
       DEBUG_PRINT(pic_struct);
       // Use this to set "DeltaTfiDivisor" (which is used to compute the frame rate):
-      double prevDeltaTfiDivisor = DeltaTfiDivisor; 
       if (fHNumber == 264) {
 	DeltaTfiDivisor =
 	  pic_struct == 0 ? 2.0 :
@@ -919,16 +955,22 @@ void H264or5VideoStreamParser
 	  pic_struct <= 12 ? 1.0 :
 	  2.0;
       }
-      // If "DeltaTfiDivisor" has changed, and we've already computed the frame rate, then
-      // adjust it, based on the new value of "DeltaTfiDivisor":
-      if (DeltaTfiDivisor != prevDeltaTfiDivisor && fParsedFrameRate != 0.0) {
-	  usingSource()->fFrameRate = fParsedFrameRate
-	    = fParsedFrameRate*(prevDeltaTfiDivisor/DeltaTfiDivisor);
-#ifdef DEBUG
-	  fprintf(stderr, "Changed frame rate to %f fps\n", usingSource()->fFrameRate);
-#endif
+    } else {
+      if (fHNumber == 264) {
+	// Need to get field_pic_flag from slice_header to set this properly! #####
+      } else { // H.265
+	DeltaTfiDivisor = 1.0;
       }
     }
+    // If "DeltaTfiDivisor" has changed, and we've already computed the frame rate, then
+    // adjust it, based on the new value of "DeltaTfiDivisor":
+    if (DeltaTfiDivisor != prevDeltaTfiDivisor && fParsedFrameRate != 0.0) {
+      usingSource()->fFrameRate = fParsedFrameRate
+	= fParsedFrameRate*(prevDeltaTfiDivisor/DeltaTfiDivisor);
+#ifdef DEBUG
+      fprintf(stderr, "Changed frame rate to %f fps\n", usingSource()->fFrameRate);
+#endif
+    }
     // Ignore the rest of the payload (timestamps) for now... #####
   }
 }
@@ -940,8 +982,6 @@ void H264or5VideoStreamParser::flushInput() {
   StreamParser::flushInput();
 }
 
-#define NUM_NEXT_SLICE_HEADER_BYTES_TO_ANALYZE 12
-
 unsigned H264or5VideoStreamParser::parse() {
   try {
     // The stream must start with a 0x00000001:
