@@ -14,20 +14,24 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2018 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up a H.265 Video Elementary Stream into NAL units.
 // Implementation
 
 #include "H265VideoStreamFramer.hh"
 
 H265VideoStreamFramer* H265VideoStreamFramer
-::createNew(UsageEnvironment& env, FramedSource* inputSource, Boolean includeStartCodeInOutput) {
-  return new H265VideoStreamFramer(env, inputSource, True, includeStartCodeInOutput);
+::createNew(UsageEnvironment& env, FramedSource* inputSource,
+	    Boolean includeStartCodeInOutput, Boolean insertAccessUnitDelimiters) {
+  return new H265VideoStreamFramer(env, inputSource, True,
+				   includeStartCodeInOutput, insertAccessUnitDelimiters);
 }
 
 H265VideoStreamFramer
-::H265VideoStreamFramer(UsageEnvironment& env, FramedSource* inputSource, Boolean createParser, Boolean includeStartCodeInOutput)
-  : H264or5VideoStreamFramer(265, env, inputSource, createParser, includeStartCodeInOutput) {
+::H265VideoStreamFramer(UsageEnvironment& env, FramedSource* inputSource, Boolean createParser,
+			Boolean includeStartCodeInOutput, Boolean insertAccessUnitDelimiters)
+  : H264or5VideoStreamFramer(265, env, inputSource, createParser,
+			     includeStartCodeInOutput, insertAccessUnitDelimiters) {
 }
 
 H265VideoStreamFramer::~H265VideoStreamFramer() {
