@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2017 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
 // RTCP
 // Implementation
 
@@ -24,7 +24,6 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #if defined(__WIN32__) || defined(_WIN32) || defined(_QNX4)
 #define snprintf _snprintf
 #endif
-#define HACK_FOR_CHROME_WEBRTC_BUG 1 //#####@@@@@
 
 ////////// RTCPMemberDatabase //////////
 
@@ -133,7 +132,7 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
     fLastSentSize(0), fLastReceivedSize(0), fLastReceivedSSRC(0),
     fTypeOfEvent(EVENT_UNKNOWN), fTypeOfPacket(PACKET_UNKNOWN_TYPE),
     fHaveJustSentPacket(False), fLastPacketSentSize(0),
-    fByeHandlerTask(NULL), fByeHandlerClientData(NULL),
+    fByeHandlerTask(NULL), fByeWithReasonHandlerTask(NULL), fByeHandlerClientData(NULL),
     fSRHandlerTask(NULL), fSRHandlerClientData(NULL),
     fRRHandlerTask(NULL), fRRHandlerClientData(NULL),
     fSpecificRRHandlerTable(NULL),
@@ -282,6 +281,15 @@ unsigned RTCPInstance::numMembers() const {
 void RTCPInstance::setByeHandler(TaskFunc* handlerTask, void* clientData,
 				 Boolean handleActiveParticipantsOnly) {
   fByeHandlerTask = handlerTask;
+  fByeWithReasonHandlerTask = NULL;
+  fByeHandlerClientData = clientData;
+  fByeHandleActiveParticipantsOnly = handleActiveParticipantsOnly;
+}
+
+void RTCPInstance::setByeWithReasonHandler(ByeWithReasonHandlerFunc* handlerTask, void* clientData,
+					   Boolean handleActiveParticipantsOnly) {
+  fByeHandlerTask = NULL;
+  fByeWithReasonHandlerTask = handlerTask;
   fByeHandlerClientData = clientData;
   fByeHandleActiveParticipantsOnly = handleActiveParticipantsOnly;
 }
@@ -492,6 +500,7 @@ void RTCPInstance
 			int tcpSocketNum, unsigned char tcpStreamChannelId) {
   do {
     Boolean callByeHandler = False;
+    char* reason = NULL; // by default, unless/until a BYE packet with a 'reason' arrives
     unsigned char* pkt = fInBuf;
 
 #ifdef DEBUG
@@ -617,11 +626,34 @@ void RTCPInstance
 	}
         case RTCP_PT_BYE: {
 #ifdef DEBUG
-	  fprintf(stderr, "BYE\n");
+	  fprintf(stderr, "BYE");
+#endif
+	  // Check whether there was a 'reason for leaving':
+	  if (length > 0) {
+	    u_int8_t reasonLength = *pkt;
+	    if (reasonLength > length-1) {
+	      // The 'reason' length field is too large!
+#ifdef DEBUG
+	      fprintf(stderr, "\nError: The 'reason' length %d is too large (it should be <= %d)\n",
+		      reasonLength, length-1);
+#endif
+	      reasonLength = length-1;
+	    }
+	    reason = new char[reasonLength + 1];
+	    for (unsigned i = 0; i < reasonLength; ++i) {
+	      reason[i] = pkt[1+i];
+	    }
+	    reason[reasonLength] = '\0';
+#ifdef DEBUG
+	    fprintf(stderr, " (reason:%s)", reason);
+#endif
+	  }
+#ifdef DEBUG
+	  fprintf(stderr, "\n");
 #endif
 	  // If a 'BYE handler' was set, arrange for it to be called at the end of this routine.
 	  // (Note: We don't call it immediately, in case it happens to cause "this" to be deleted.)
-	  if (fByeHandlerTask != NULL
+	  if ((fByeHandlerTask != NULL || fByeWithReasonHandlerTask != NULL)
 	      && (!fByeHandleActiveParticipantsOnly
 		  || (fSource != NULL
 		      && fSource->receptionStatsDB().lookup(reportSenderSSRC) != NULL)
@@ -830,10 +862,17 @@ void RTCPInstance
     onReceive(typeOfPacket, totPacketSize, reportSenderSSRC);
 
     // Finally, if we need to call a "BYE" handler, do so now (in case it causes "this" to get deleted):
-    if (callByeHandler && fByeHandlerTask != NULL/*sanity check*/) {
-      TaskFunc* byeHandler = fByeHandlerTask;
-      fByeHandlerTask = NULL; // because we call the handler only once, by default
-      (*byeHandler)(fByeHandlerClientData);
+    if (callByeHandler) {
+      if (fByeHandlerTask != NULL) { // call a BYE handler without including a 'reason'
+	TaskFunc* byeHandler = fByeHandlerTask;
+	fByeHandlerTask = NULL; // because we call the handler only once, by default
+	(*byeHandler)(fByeHandlerClientData);
+      } else if (fByeWithReasonHandlerTask != NULL) { // call a BYE handler that includes a 'reason'
+	ByeWithReasonHandlerFunc* byeHandler = fByeWithReasonHandlerTask;
+	fByeWithReasonHandlerTask = NULL; // because we call the handler only once, by default
+	(*byeHandler)(fByeHandlerClientData, reason);
+	    // Note that the handler function is responsible for delete[]ing "reason"
+      }
     }
   } while (0);
 }
@@ -878,14 +917,18 @@ void RTCPInstance::sendReport() {
   }
 }
 
-void RTCPInstance::sendBYE() {
+void RTCPInstance::sendBYE(char const* reason) {
 #ifdef DEBUG
-  fprintf(stderr, "sending BYE\n");
+  if (reason != NULL) {
+    fprintf(stderr, "sending BYE (reason:%s)\n", reason);
+  } else {
+    fprintf(stderr, "sending BYE\n");
+  }
 #endif
   // The packet must begin with a SR and/or RR report:
   (void)addReport(True);
 
-  addBYE();
+  addBYE(reason);
   sendBuiltPacket();
 }
 
@@ -1125,10 +1168,20 @@ void RTCPInstance::addSDES() {
   while (numPaddingBytesNeeded-- > 0) fOutBuf->enqueue(&zero, 1);
 }
 
-void RTCPInstance::addBYE() {
-  unsigned rtcpHdr = 0x81000000; // version 2, no padding, 1 SSRC
+void RTCPInstance::addBYE(char const* reason) {
+  u_int32_t rtcpHdr = 0x81000000; // version 2, no padding, 1 SSRC
   rtcpHdr |= (RTCP_PT_BYE<<16);
-  rtcpHdr |= 1; // 2 32-bit words total (i.e., with 1 SSRC)
+  u_int16_t num32BitWords = 2; // by default, two 32-bit words total (i.e., with 1 SSRC)
+  u_int8_t reasonLength8Bits = 0; // by default
+  if (reason != NULL) {
+    // We need to add more 32-bit words for the 'length+reason':
+    unsigned const reasonLength = strlen(reason);
+    reasonLength8Bits = reasonLength < 0xFF ? (u_int8_t)reasonLength : 0xFF;
+    unsigned numExtraWords = ((1/*reason length field*/+reasonLength8Bits)+3)/4;
+    
+    num32BitWords += numExtraWords;
+  }
+  rtcpHdr |= (num32BitWords-1); // length field
   fOutBuf->enqueueWord(rtcpHdr);
 
   if (fSource != NULL) {
@@ -1136,6 +1189,31 @@ void RTCPInstance::addBYE() {
   } else if (fSink != NULL) {
     fOutBuf->enqueueWord(fSink->SSRC());
   }
+
+  num32BitWords -= 2; // ASSERT: num32BitWords >= 0
+  if (num32BitWords > 0) {
+    // Add a length+'reason for leaving':
+    // First word:
+    u_int32_t lengthPlusFirst3ReasonBytes = reasonLength8Bits<<24;
+    unsigned index = 0;
+    if (reasonLength8Bits > index) lengthPlusFirst3ReasonBytes |= ((u_int8_t)reason[index++])<<16;
+    if (reasonLength8Bits > index) lengthPlusFirst3ReasonBytes |= ((u_int8_t)reason[index++])<<8;
+    if (reasonLength8Bits > index) lengthPlusFirst3ReasonBytes |= (u_int8_t)reason[index++];
+    fOutBuf->enqueueWord(lengthPlusFirst3ReasonBytes);
+
+    // Any subsequent words:
+    if (reasonLength8Bits > 3) {
+      // ASSERT: num32BitWords > 1
+      while (--num32BitWords > 0) {
+	u_int32_t fourMoreReasonBytes = 0;
+	if (reasonLength8Bits > index) fourMoreReasonBytes |= ((u_int8_t)reason[index++])<<24;
+	if (reasonLength8Bits > index) fourMoreReasonBytes |= ((u_int8_t)reason[index++])<<16;
+	if (reasonLength8Bits > index) fourMoreReasonBytes |= ((u_int8_t)reason[index++])<<8;
+	if (reasonLength8Bits > index) fourMoreReasonBytes |= (u_int8_t)reason[index++];
+	fOutBuf->enqueueWord(fourMoreReasonBytes);
+      }
+    }
+  }
 }
 
 void RTCPInstance::schedule(double nextTime) {
